//===========================================================================
// Module name: key_test.v
// Description: Detect the two keys KEY1 ~ KEY2 on the development board. When the key is detected, the LED light will flip
//===========================================================================
`timescale 1ns / 1ps
module key_test	 (
         clk,		  // Input clock on development board: 50Mhz
         rst_n,		  // Input reset button on development board
         key_in,		  // Input key signal (KEY1 ~ KEY2)
         led_out		  // Output LED lights for controlling LEDs on the development board
       );

//===========================================================================
// PORT declarations
//===========================================================================
input	     clk;
input	     rst_n;
input		 key_in;
output		 led_out;

//Register definition
reg [19:0] count;
reg [1:0] key_scan; //Key scan value KEY

//===========================================================================
// Sampling the key value, scanning once every 20ms, the sampling frequency is less than the key glitch frequency, which is equivalent to filtering out high frequency glitch signals.
//===========================================================================
always @(posedge clk or negedge rst_n)	   //Detects rising edge of clock and falling edge of reset
  begin
    if(!rst_n)		     //Reset signal active low
      count <= 20'd0;	     //Clear the counter to 0
    else
      begin
        if(count ==20'd999_999)   //20msScan once, 20ms count (50M / 50-1 = 999_999)
          begin
            count <= 20'b0;	   //The counter reaches 20ms, and the counter is cleared
            key_scan[0] <= key_in; //Sampling button input level
          end
        else
          count <= count + 20'b1; //Increment the counter
      end
  end
//===========================================================================
// Key signal latches a clock tick
//===========================================================================
reg [1:0] key_scan_r;
always @(posedge clk)
  key_scan_r <= key_scan;

wire [1:0] flag_key = key_scan_r[1:0] & (~key_scan[1:0]);  //When a falling edge change of a key is detected, it means that the key is pressed and the key is valid

//===========================================================================
// LEDLight control, when the button is pressed, the relevant LED output is flipped
//===========================================================================
reg temp_led;
always @ (posedge clk or negedge rst_n)	     //Detects rising edge of clock and falling edge of reset
  begin
    if (!rst_n)								//Reset signal active low
      temp_led <= 1'b1;						//LEDLight control signal output is low, LED light is off
    else
      begin
        if ( flag_key[0] )
          temp_led <= ~temp_led;	 //When the value of KEY1 is changed, the LED will turn on and off.
        if ( flag_key[1] )
          temp_led <= ~temp_led;	 //When the value of KEY2 is changed, the LED will turn on and off.
      end
  end

assign led_out = temp_led;

endmodule

